//=============================================================================
// SNH_TurningDirection.js (Ver1.1.0)
//=============================================================================
// Version
// 1.1.0 2018/12/13 向いてる方に歩く時はそのまま歩くように変更
//                  移動中の方向転換では立ち止まらないモードを追加
//                  方向転換の間足踏みするモードを追加
// 1.0.0 2018/10/25 初版
/*:
 * @plugindesc Change something.
 * @author Gabacho at SotoNoHito(http://www.star-write-dream.com)
 * @help OMG! Sory. I can not write a English. Because I am Japanese! 
 * This plugin is released under the MIT License.
 *
 * @param waitTime
 * @type number
 * @desc KeyHold time for move starting.(frame)
 * @min 0
 * @default 20
 *
 * @param dashCancel
 * @type string
 * @desc Disabled Function when dashing.(true/false)
 * @default true
 *
 * @param stompMode
 * @type string
 * @desc Stepping on during the change of direction.(true/false)
 * @default true
 *
 * @param stompSpeed
 * @type number
 * @desc Foot speed, Limited to stompMode.(true/false)
 * @default 0
 *
 * @param smoothMode
 * @type string
 * @desc non stop. when turning direction, when player already moving.(true/false)
 * @default true
 *
 * @param smoothTime
 * @type number
 * @desc Even if this time input is not done, Handling while moving.(frame)
 * @default 10
 *
*/
 
/*:ja
 * @plugindesc 方向転換（長押し移動）プラグイン
 * @author ガバチョ（そとの人）(http://www.star-write-dream.com)
 * @help プラグインコマンドはありません。
 * このプラグインはMITライセンスです。
 *  ----------------------------------------------------------------------------
 *
 * 方向キーを短く押すとキャラの向きだけ変わります
 * 方向キーを長押しするとキャラが移動します。
 * ただし、向いている方にそのまま歩く時はすぐ移動します。
 * 
 * ※画面のクリック・タップによる移動には効果がありませんのでご注意ください。
 *
 * ----------------------------------------------------------------------------
 * パラメータ
 * 
 * ■waitTime（初期値:20）
 * 移動を開始するまでのフレーム数（長押しの時間）を指定してください。
 * 長いとイライラしますし、短いと向きだけ変える操作がむずかしくなります。
 * 
 * ■dashCancel（初期値:true）
 * true:ダッシュ中はこの機能を無効にします。
 *      （長押ししなくてもすぐに移動します）
 * false:ダッシュ中も長押しが必要です。
 * 
 * ■stompMode（初期値:true）
 * true:waitTimeの間、足踏みします。
 * false:足踏みしません
 * 
 * ■stompSpeed（初期値:0）
 * ※stompModeがtrueの時だけ影響します。
 * 足踏みのスピードを指定してください。
 * 数値は「移動速度の変更」と同じです。
 * 1:1/8倍速～4:標準～6:4倍速
 * 0だと移動スピードと同じです。
 * 7以降も反映されますが、エディタからは設定できない数値なので自己責任で。
 * 
 * ■smoothMode（初期値:true）
 * true:「移動中」の方向転換では長押し判定しません。
 * false:方向転換する時はかならず長押し判定します。
 * 
 * ■smoothTime（初期値:10）
 * ※smoothModeがtrueの時だけ影響します。
 * 立ち止まってからこの時間は「移動中」扱いです。
 * フレーム数で設定してください。
 * 
 * @param waitTime
 * @type number
 * @desc 移動を開始するまでのフレーム数（初期値:20）
 * @min 0
 * @default 20
 *
 * @param dashCancel
 * @type string
 * @desc ダッシュ中は機能を無効にする（true/false）
 * @default true
 *
 * @param stompMode
 * @type string
 * @desc 方向転換してから歩き出すまで足踏みする（true/false）
 * @default true
 * 
 * @param stompSpeed
 * @type number
 * @desc 足踏みのスピード（0は移動スピードと同じ）
 * @default 0
 * 
 * @param smoothMode
 * @type string
 * @desc 「移動中」の方向転換では立ち止まらない（true/false）
 * @default true
 * 
 * @param smoothTime
 * @type number
 * @desc 立ち止まってからも「移動中」扱いとするフレーム数（初期値：10）
 * @default 10
*/

(function() {

    var parameters = PluginManager.parameters('SNH_TurningDirection');
    var waitTime = (parameters['waitTime'] || 20);
    var isDashCancel = Boolean(parameters['dashCancel']  === 'true' || false);
    var isSmooth = Boolean(parameters['smoothMode']  === 'true' || false);
    var isStomp = Boolean(parameters['stompMode']  === 'true' || false);
    var stompSpeed = Number((parameters['stompSpeed'] || 0));
    var smoothTime = (parameters['smoothTime'] || 20);
    var beforeDirection = 0;
    var moveStart=false;
    var moveEndCnt = 0;
    var beforeMoveSpeed = 4;

    //リピート判定を簡略化
    Input.isRepeatedEX = function(keyName) {
    if ((isDashCancel && $gamePlayer.isDashing())
        || (this._isEscapeCompatible(keyName) && this.isRepeated('escape'))) {
        return true;
    } else {
        return this._pressedTime >= waitTime;
    }
    };
    
    //再定義
    Game_Player.prototype.moveByInput = function() {
            if (!this.isMoving() && this.canMove()) {
            var direction = this.getInputDirection();
            if (direction > 0) {
                $gameTemp.clearDestination();
            } else if ($gameTemp.isDestinationValid()){
                var x = $gameTemp.destinationX();
                var y = $gameTemp.destinationY();
                direction = this.findDirectionTo(x, y);
            }
            if (direction > 0) {
                //同じ方向にはすぐ歩く
                if(direction === beforeDirection){
                    if (!moveStart){
                        moveEndCnt = 0;
                    }
                    moveStart=true;
                    this.executeMove(direction);
                    return;
                }
                
                if (isSmooth){
                    //移動中だったら長押し判定しない
                    if (moveEndCnt <= smoothTime){
                        this.executeMove(direction);
                        moveEndCnt = 0;
                        return;
                    }
                }

                //長押し判定
                if (Input.isRepeatedEX('left')
                  ||Input.isRepeatedEX('right')
                  ||Input.isRepeatedEX('up')
                  ||Input.isRepeatedEX('down')
                ) {
                    moveStart = true;
                    moveEndCnt = 0;
                    this.executeMove(direction);
                } else {
                    //方向転換
                    $gamePlayer.setDirection(direction);
                    beforeDirection = $gamePlayer.direction();
                    
                    if(isStomp) {
                        //足踏みあり
                        if (stompSpeed == 0) {
                            //足踏みスピード指定なし
                            $gamePlayer.forceMoveRoute({
                                "list":[
                                    {"code":33},
                                    {"code":15, "parameters":[waitTime]},
                                    {"code":34},
                                    {"code":0}],
                                "repeat":false,
                                "skippable":true})
                        } else {
                            //足踏みスピード指定あり
                            beforeMoveSpeed = $gamePlayer._moveSpeed;
                            $gamePlayer.forceMoveRoute({
                                "list":[
                                    {"code":29, "parameters":[stompSpeed]},
                                    {"code":33},
                                    {"code":15, "parameters":[waitTime]},
                                    {"code":29, "parameters":[beforeMoveSpeed]},
                                    {"code":34},
                                    {"code":0}],
                                "repeat":false,
                                "skippable":true})
                        }
                    } else {
                        //足踏みなし
                        $gamePlayer.forceMoveRoute({
                                "list":[
                                    {"code":15, "parameters":[waitTime]},
                                    {"code":0}],
                                "repeat":false,
                                "skippable":true})
                    }
                }
            }
        }
        
    };

    //停止時の処理を追加
    var _Game_Player_updateNonmoving=Game_Player.prototype.updateNonmoving;
    Game_Player.prototype.updateNonmoving = function(wasMoving) {
        _Game_Player_updateNonmoving.call(this, wasMoving);
        if(!wasMoving){
            moveStart = false;
            beforeDirection = $gamePlayer.direction();
            if(isSmooth && moveEndCnt <= smoothTime){
                moveEndCnt++;
            }
        }
    }
    
})();
